@@ -1,4 +1,4 @@
-use std::{mem, ptr::NonNull, time::Duration, u8};
+use std::{mem, fmt::{self, Debug}, ptr::NonNull, time::Duration, u8};
 
 use libc::{c_int, c_uchar, c_uint};
 use libusb1_sys::{constants::*, *};
@@ -107,7 +107,7 @@ impl<'a> Iterator for ClaimedInterfacesIter<'a> {
 #[derive(Eq, PartialEq)]
 pub struct DeviceHandle<T: UsbContext> {
     context: T,
-    handle: NonNull<libusb_device_handle>,
+    handle: Option<NonNull<libusb_device_handle>>,
     interfaces: ClaimedInterfaces,
 }
 
@@ -116,10 +116,12 @@ impl<T: UsbContext> Drop for DeviceHandle<T> {
     fn drop(&mut self) {
         unsafe {
             for iface in self.interfaces.iter() {
-                libusb_release_interface(self.handle.as_ptr(), iface as c_int);
+                libusb_release_interface(self.as_raw(), iface as c_int);
             }
 
-            libusb_close(self.handle.as_ptr());
+            if let Some(handle) = self.handle {
+                libusb_close(handle.as_ptr());
+            }
         }
     }
 }
@@ -127,13 +129,41 @@ impl<T: UsbContext> Drop for DeviceHandle<T> {
 unsafe impl<T: UsbContext> Send for DeviceHandle<T> {}
 unsafe impl<T: UsbContext> Sync for DeviceHandle<T> {}
 
+impl<T: UsbContext> Debug for DeviceHandle<T> {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        f.debug_struct("DeviceHandle")
+            .field("device", &self.device())
+            .field("handle", &self.handle)
+            .field("interfaces", &self.interfaces)
+            .finish()
+    }
+}
+
 impl<T: UsbContext> DeviceHandle<T> {
     /// Get the raw libusb_device_handle pointer, for advanced use in unsafe code.
     ///
     /// This structure tracks claimed interfaces, and will get out if sync if interfaces are
     /// manipulated externally. Use only libusb endpoint IO functions.
     pub fn as_raw(&self) -> *mut libusb_device_handle {
-        self.handle.as_ptr()
+        // Safety: handle is Some() after initialization
+        match self.handle {
+            Some(it) => it.as_ptr(),
+            _ => unreachable!(),
+        }
+    }
+
+    /// Consumes the `DeviceHandle`, returning the raw libusb_device_handle
+    /// pointer, for advanced use in unsafe code.
+    ///
+    /// # Safety
+    ///
+    /// Panics if you have any claimed interfaces on this handle.
+    pub fn into_raw(mut self) -> *mut libusb_device_handle {
+        assert_eq!(self.interfaces.size(), 0);
+        match self.handle.take() {
+            Some(it) => it.as_ptr(),
+            _ => unreachable!(),
+        }
     }
 
     /// Get the context associated with this device
@@ -146,7 +176,7 @@ impl<T: UsbContext> DeviceHandle<T> {
         unsafe {
             device::Device::from_libusb(
                 self.context.clone(),
-                std::ptr::NonNull::new_unchecked(libusb_get_device(self.handle.as_ptr())),
+                std::ptr::NonNull::new_unchecked(libusb_get_device(self.as_raw())),
             )
         }
     }
@@ -161,7 +191,7 @@ impl<T: UsbContext> DeviceHandle<T> {
     ) -> DeviceHandle<T> {
         DeviceHandle {
             context,
-            handle,
+            handle: Some(handle),
             interfaces: ClaimedInterfaces::new(),
         }
     }
@@ -171,7 +201,7 @@ impl<T: UsbContext> DeviceHandle<T> {
         let mut config = mem::MaybeUninit::<c_int>::uninit();
 
         try_unsafe!(libusb_get_configuration(
-            self.handle.as_ptr(),
+            self.as_raw(),
             config.as_mut_ptr()
         ));
         Ok(unsafe { config.assume_init() } as u8)
@@ -180,7 +210,7 @@ impl<T: UsbContext> DeviceHandle<T> {
     /// Sets the device's active configuration.
     pub fn set_active_configuration(&mut self, config: u8) -> crate::Result<()> {
         try_unsafe!(libusb_set_configuration(
-            self.handle.as_ptr(),
+            self.as_raw(),
             c_int::from(config)
         ));
         Ok(())
@@ -188,19 +218,19 @@ impl<T: UsbContext> DeviceHandle<T> {
 
     /// Puts the device in an unconfigured state.
     pub fn unconfigure(&mut self) -> crate::Result<()> {
-        try_unsafe!(libusb_set_configuration(self.handle.as_ptr(), -1));
+        try_unsafe!(libusb_set_configuration(self.as_raw(), -1));
         Ok(())
     }
 
     /// Resets the device.
     pub fn reset(&mut self) -> crate::Result<()> {
-        try_unsafe!(libusb_reset_device(self.handle.as_ptr()));
+        try_unsafe!(libusb_reset_device(self.as_raw()));
         Ok(())
     }
 
     /// Clear the halt/stall condition for an endpoint.
     pub fn clear_halt(&mut self, endpoint: u8) -> crate::Result<()> {
-        try_unsafe!(libusb_clear_halt(self.handle.as_ptr(), endpoint));
+        try_unsafe!(libusb_clear_halt(self.as_raw(), endpoint));
         Ok(())
     }
 
@@ -208,7 +238,7 @@ impl<T: UsbContext> DeviceHandle<T> {
     ///
     /// This method is not supported on all platforms.
     pub fn kernel_driver_active(&self, iface: u8) -> crate::Result<bool> {
-        match unsafe { libusb_kernel_driver_active(self.handle.as_ptr(), c_int::from(iface)) } {
+        match unsafe { libusb_kernel_driver_active(self.as_raw(), c_int::from(iface)) } {
             0 => Ok(false),
             1 => Ok(true),
             err => Err(error::from_libusb(err)),
@@ -220,7 +250,7 @@ impl<T: UsbContext> DeviceHandle<T> {
     /// This method is not supported on all platforms.
     pub fn detach_kernel_driver(&mut self, iface: u8) -> crate::Result<()> {
         try_unsafe!(libusb_detach_kernel_driver(
-            self.handle.as_ptr(),
+            self.as_raw(),
             c_int::from(iface)
         ));
         Ok(())
@@ -231,7 +261,7 @@ impl<T: UsbContext> DeviceHandle<T> {
     /// This method is not supported on all platforms.
     pub fn attach_kernel_driver(&mut self, iface: u8) -> crate::Result<()> {
         try_unsafe!(libusb_attach_kernel_driver(
-            self.handle.as_ptr(),
+            self.as_raw(),
             c_int::from(iface)
         ));
         Ok(())
@@ -248,7 +278,7 @@ impl<T: UsbContext> DeviceHandle<T> {
     /// this function was never called.
     pub fn set_auto_detach_kernel_driver(&mut self, auto_detach: bool) -> crate::Result<()> {
         try_unsafe!(libusb_set_auto_detach_kernel_driver(
-            self.handle.as_ptr(),
+            self.as_raw(),
             auto_detach.into()
         ));
         Ok(())
@@ -260,7 +290,7 @@ impl<T: UsbContext> DeviceHandle<T> {
     /// when the device handle goes out of scope.
     pub fn claim_interface(&mut self, iface: u8) -> crate::Result<()> {
         try_unsafe!(libusb_claim_interface(
-            self.handle.as_ptr(),
+            self.as_raw(),
             c_int::from(iface)
         ));
         self.interfaces.insert(iface);
@@ -270,7 +300,7 @@ impl<T: UsbContext> DeviceHandle<T> {
     /// Releases a claimed interface.
     pub fn release_interface(&mut self, iface: u8) -> crate::Result<()> {
         try_unsafe!(libusb_release_interface(
-            self.handle.as_ptr(),
+            self.as_raw(),
             c_int::from(iface)
         ));
         self.interfaces.remove(iface);
@@ -280,7 +310,7 @@ impl<T: UsbContext> DeviceHandle<T> {
     /// Sets an interface's active setting.
     pub fn set_alternate_setting(&mut self, iface: u8, setting: u8) -> crate::Result<()> {
         try_unsafe!(libusb_set_interface_alt_setting(
-            self.handle.as_ptr(),
+            self.as_raw(),
             c_int::from(iface),
             c_int::from(setting)
         ));
@@ -291,7 +321,7 @@ impl<T: UsbContext> DeviceHandle<T> {
     ///
     /// This function attempts to read from the interrupt endpoint with the address given by the
     /// `endpoint` parameter and fills `buf` with any data received from the endpoint. The function
-    /// blocks up to the amount of time specified by `timeout`.
+    /// blocks up to the amount of time specified by `timeout`. Minimal `timeout` is 1 microseconds.
     ///
     /// If the return value is `Ok(n)`, then `buf` is populated with `n` bytes of data received
     /// from the endpoint.
@@ -321,7 +351,7 @@ impl<T: UsbContext> DeviceHandle<T> {
         let mut transferred = mem::MaybeUninit::<c_int>::uninit();
         unsafe {
             match libusb_interrupt_transfer(
-                self.handle.as_ptr(),
+                self.as_raw(),
                 endpoint,
                 buf.as_mut_ptr() as *mut c_uchar,
                 buf.len() as c_int,
@@ -346,7 +376,7 @@ impl<T: UsbContext> DeviceHandle<T> {
     ///
     /// This function attempts to write the contents of `buf` to the interrupt endpoint with the
     /// address given by the `endpoint` parameter. The function blocks up to the amount of time
-    /// specified by `timeout`.
+    /// specified by `timeout`. Minimal `timeout` is 1 microseconds.
     ///
     /// If the return value is `Ok(n)`, then `n` bytes of `buf` were written to the endpoint.
     ///
@@ -374,7 +404,7 @@ impl<T: UsbContext> DeviceHandle<T> {
         let mut transferred = mem::MaybeUninit::<c_int>::uninit();
         unsafe {
             match libusb_interrupt_transfer(
-                self.handle.as_ptr(),
+                self.as_raw(),
                 endpoint,
                 buf.as_ptr() as *mut c_uchar,
                 buf.len() as c_int,
@@ -399,7 +429,7 @@ impl<T: UsbContext> DeviceHandle<T> {
     ///
     /// This function attempts to read from the bulk endpoint with the address given by the
     /// `endpoint` parameter and fills `buf` with any data received from the endpoint. The function
-    /// blocks up to the amount of time specified by `timeout`.
+    /// blocks up to the amount of time specified by `timeout`. Minimal `timeout` is 1 microseconds.
     ///
     /// If the return value is `Ok(n)`, then `buf` is populated with `n` bytes of data received
     /// from the endpoint.
@@ -429,7 +459,7 @@ impl<T: UsbContext> DeviceHandle<T> {
         let mut transferred = mem::MaybeUninit::<c_int>::uninit();
         unsafe {
             match libusb_bulk_transfer(
-                self.handle.as_ptr(),
+                self.as_raw(),
                 endpoint,
                 buf.as_mut_ptr() as *mut c_uchar,
                 buf.len() as c_int,
@@ -454,7 +484,7 @@ impl<T: UsbContext> DeviceHandle<T> {
     ///
     /// This function attempts to write the contents of `buf` to the bulk endpoint with the address
     /// given by the `endpoint` parameter. The function blocks up to the amount of time specified
-    /// by `timeout`.
+    /// by `timeout`. Minimal `timeout` is 1 microseconds.
     ///
     /// If the return value is `Ok(n)`, then `n` bytes of `buf` were written to the endpoint.
     ///
@@ -477,7 +507,7 @@ impl<T: UsbContext> DeviceHandle<T> {
         let mut transferred = mem::MaybeUninit::<c_int>::uninit();
         unsafe {
             match libusb_bulk_transfer(
-                self.handle.as_ptr(),
+                self.as_raw(),
                 endpoint,
                 buf.as_ptr() as *mut c_uchar,
                 buf.len() as c_int,
@@ -502,7 +532,7 @@ impl<T: UsbContext> DeviceHandle<T> {
     ///
     /// This function attempts to read data from the device using a control transfer and fills
     /// `buf` with any data received during the transfer. The function blocks up to the amount of
-    /// time specified by `timeout`.
+    /// time specified by `timeout`. Minimal `timeout` is 1 microseconds.
     ///
     /// The parameters `request_type`, `request`, `value`, and `index` specify the fields of the
     /// control transfer setup packet (`bmRequestType`, `bRequest`, `wValue`, and `wIndex`
@@ -539,7 +569,7 @@ impl<T: UsbContext> DeviceHandle<T> {
         }
         let res = unsafe {
             libusb_control_transfer(
-                self.handle.as_ptr(),
+                self.as_raw(),
                 request_type,
                 request,
                 value,
@@ -561,6 +591,7 @@ impl<T: UsbContext> DeviceHandle<T> {
     ///
     /// This function attempts to write the contents of `buf` to the device using a control
     /// transfer. The function blocks up to the amount of time specified by `timeout`.
+    /// Minimal `timeout` is 1 microseconds.
     ///
     /// The parameters `request_type`, `request`, `value`, and `index` specify the fields of the
     /// control transfer setup packet (`bmRequestType`, `bRequest`, `wValue`, and `wIndex`
@@ -597,7 +628,7 @@ impl<T: UsbContext> DeviceHandle<T> {
         }
         let res = unsafe {
             libusb_control_transfer(
-                self.handle.as_ptr(),
+                self.as_raw(),
                 request_type,
                 request,
                 value,
@@ -658,7 +689,7 @@ impl<T: UsbContext> DeviceHandle<T> {
         let capacity = buf.capacity() as i32;
 
         let res = unsafe {
-            libusb_get_string_descriptor_ascii(self.handle.as_ptr(), index, ptr, capacity)
+            libusb_get_string_descriptor_ascii(self.as_raw(), index, ptr, capacity)
         };
 
         if res < 0 {
@@ -681,32 +712,41 @@ impl<T: UsbContext> DeviceHandle<T> {
         index: u8,
         timeout: Duration,
     ) -> crate::Result<String> {
-        let mut buf = [0u8; 255];
+        let mut buf = [0u16; 128];
 
-        let len = self.read_control(
-            request_type(Direction::In, RequestType::Standard, Recipient::Device),
-            LIBUSB_REQUEST_GET_DESCRIPTOR,
-            u16::from(LIBUSB_DT_STRING) << 8 | u16::from(index),
-            language.lang_id(),
-            &mut buf,
-            timeout,
-        )?;
+        let len = {
+            // SAFETY: since we create slice from existing slice pointer valid
+            // alignment of [u8] less or equal to the [u16]
+            // size is less then allocated buffer (128 * 2 = 256 => 256 < 255)
+            let buf = unsafe {
+                std::slice::from_raw_parts_mut(
+                    buf.as_mut_ptr().cast::<u8>(),
+                    255, // Some devices choke on size > 255
+                )
+            };
 
-        if len < 2 || buf[0] != len as u8 || len & 0x01 != 0 {
-            return Err(Error::BadDescriptor);
-        }
+            let len = self.read_control(
+                request_type(Direction::In, RequestType::Standard, Recipient::Device),
+                LIBUSB_REQUEST_GET_DESCRIPTOR,
+                u16::from(LIBUSB_DT_STRING) << 8 | u16::from(index),
+                language.lang_id(),
+                buf,
+                timeout,
+            )?;
+
+            if len < 2 || buf[0] != len as u8 || len & 0x01 != 0 {
+                return Err(Error::BadDescriptor);
+            }
+
+            len
+        };
 
         if len == 2 {
             return Ok(String::new());
         }
 
-        let utf16: Vec<u16> = buf[..len]
-            .chunks(2)
-            .skip(1)
-            .map(|chunk| u16::from(chunk[0]) | u16::from(chunk[1]) << 8)
-            .collect();
-
-        String::from_utf16(&utf16).map_err(|_| Error::Other)
+        // len in bytes, skip first element(it's contain descriptor type and len)
+        String::from_utf16(&buf[1..(len / 2)]).map_err(|_| Error::Other)
     }
 
     /// Reads the device's manufacturer string descriptor (ascii).
